<?php
/**
 * @package WordPress
 * @subpackage SaaS_I_Theme
 *
 * Copyright (c) 2010 WorryFreeLabs
 *
 * This file is part of SaaS I WordPress theme
 */

require_once 'SaaS/Exception.php';
require_once 'SaaS/Options.php';
require_once 'SaaS/View.php';
require_once 'SaaS/Tour/Walker/Page.php';

/**
 * Admin-interface for SaaS WordPress theme
 */
class SaaS
{
	/**
	 * Theme version
	 */
	const VERSION = '1.4';
	
	/**
	 * The lookup key used to locate the options record in the wp_options table
	 */
	const OPTIONS_KEY = 'saas-options';
	
	/**
	 * The hook to be used in all variable actions and filters
	 */
	const HOOK = 'saas';
	
	/**
	 * Singleton instance
	 * @var SaaS
	 */
	protected static $_instance = null;
	
	/**
	 * An instance of the options structure containing all options for this theme
	 * @var SaaS_Options
	 */
	protected $_options = null;
	
	/**
	 * View object
	 * @var SaaS_View
	 */
	protected $_view = null;
	
	/**
	 * Singleton pattern implementation makes "new" unavailable
	 *
	 * @return void
	 */
	protected function __construct()
	{}
	
	/**
	 * Singleton pattern implementation makes "clone" unavailable
	 *
	 * @return void
	 */
	protected function __clone()
	{}
	
	/**
	 * Returns an instance of SaaS
	 *
	 * Singleton pattern implementation
	 *
	 * @return SaaS
	 */
	public static function getInstance()
	{
		if (null === self::$_instance)
		{
			self::$_instance = new self();
			self::$_instance->_options = new SaaS_Options(self::OPTIONS_KEY);
			self::$_instance->_view = new SaaS_View();
		}
		
		return self::$_instance;
	}
	
	/**
	 * Initializes singleton instance and assigns hooks callbacks
	 *
	 * @param string $bootstrapFile The full path to the theme bootstrap file
	 * @return SaaS
	 */
	public static function init($bootstrapFile)
	{
		$instance = self::getInstance();
		
		// Activation
		if (is_admin() && isset($_GET['activated']) && isset($GLOBALS['pagenow']) && $GLOBALS['pagenow'] == 'themes.php')
		{
			$instance->activate();
		}
		
		// Admin hooks and action callbacks
		if (is_admin())
		{
			add_action('admin_menu', array($instance, 'registerOptionsPages'));
			add_action('in_admin_footer', array($instance, 'renderAdminFooter'));
			add_action(self::HOOK . '-saveHomepageOptions', array($instance, 'saveHomepageOptions'));
			add_action(self::HOOK . '-savePricingGridOptions', array($instance, 'savePricingGridOptions'));
			add_action(self::HOOK . '-saveGeneralOptions', array($instance, 'saveGeneralOptions'));
			add_action(self::HOOK . '-saveFooterOptions', array($instance, 'saveFooterOptions'));
		}
		else
		{
			// Shortcodes
			add_shortcode('saas-button', array($instance, 'shortcodeButton'));
			add_shortcode('saas-link', array($instance, 'shortcodeLink'));
			add_shortcode('saas-heading', array($instance, 'shortcodeHeading'));
			add_shortcode('saas-home', array($instance, 'shortcodeHome'));
			add_shortcode('saas-featured-image', array($instance, 'shortcodeFeaturedImage'));
			add_shortcode('saas-features', array($instance, 'shortcodeFeatures'));
			add_shortcode('saas-feature', array($instance, 'shortcodeFeature'));
			add_shortcode('saas-testimonials', array($instance, 'shortcodeTestimonials'));
			add_shortcode('saas-testimonial', array($instance, 'shortcodeTestimonial'));
			add_shortcode('saas-free-trial', array($instance, 'shortcodeFreeTrial'));
			add_shortcode('saas-pricing-grid', array($instance, 'shortcodePricingGrid'));
			add_shortcode('saas-grid-column', array($instance, 'shortcodeGridColumn'));
			add_shortcode('saas-grid-option', array($instance, 'shortcodeGridOption'));
			add_shortcode('saas-footer-links1', array($instance, 'shortcodeFooterLinks1'));
			add_shortcode('saas-footer-links2', array($instance, 'shortcodeFooterLinks2'));
			add_shortcode('saas-footer-links3', array($instance, 'shortcodeFooterLinks3'));
			add_shortcode('saas-social-links', array($instance, 'shortcodeSocialLinks'));
			add_shortcode('saas-contact-information', array($instance, 'shortcodeContactInformation'));
			add_shortcode('saas-contact-line', array($instance, 'shortcodeContactLine'));
			add_shortcode('saas-try-us', array($instance, 'shortcodeTryUs'));
			add_shortcode('saas-tour-description', array($instance, 'shortcodeTourDescription'));
			add_shortcode('saas-tour-features-list', array($instance, 'shortcodeTourFeaturesList'));
			add_shortcode('saas-tour-feature', array($instance, 'shortcodeTourFeature'));
		}
		
		return $instance;
	}
	
	/**
	 * Performs installation action if necessary
	 *
	 * @return void
	 */
	public function activate()
	{
		$installedVersion = $this->_options->getOption('version');
		
		// Not installed?
		if (null === $installedVersion)
		{
			$this->_install();
		}
		// Old version?
		elseif (version_compare($installedVersion, self::VERSION, '<'))
		{
			$this->_upgrade();
		}
	}
	
	/**
	 * Performs installation when theme is activated for the first time
	 *
	 * @return void
	 */
	protected function _install()
	{
		$defaults = array
		(
			'version' => self::VERSION,
			'colorScheme' => 'default',
			'homeMessageTitle' => __('Our Web App is Amazing!', 'saas'),
			'homeMessageButton' => '[saas-button type="anchor" url="{baseUrl}/plans-pricing" text="' . __('See Plans & Pricing', 'saas') . '"]',
			'homeMessageContent' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Phasellus pretium ante vel diam imperdiet pharetra. Aenean convallis, orci ut adipiscing congue, erat sem imperdiet ipsum, quis porta mauris odio sit amet urna.',
			'homeFeaturedImages' => '[saas-featured-image title="' . __('Dashboard', 'saas') . '" image="{templateUrl}/images/screenshot.png"]
[saas-featured-image title="' . __('Feature 1', 'saas') . '" image="{templateUrl}/images/screenshot2.png"]
[saas-featured-image title="' . __('Feature 2', 'saas') . '" image="{templateUrl}/images/screenshot.png"]
[saas-featured-image title="' . __('Feature 3', 'saas') . '" image="{templateUrl}/images/screenshot2.png"]',
			'homeContentArea' => '[saas-feature title="' . __('Feature Set 1', 'saas') . '" image="{templateUrl}/images/i_calendar.png"]Lorem ipsum dolor sit amet, consectetur adipiscing elit. Pellentesque vehicula, quam sed ultrices gravida, felis odio iaculis ipsum.[/saas-feature]
[saas-feature title="' . __('Feature Set 2', 'saas') . '" image="{templateUrl}/images/i_basket.png"]Lorem ipsum dolor sit amet, consectetur adipiscing elit. Pellentesque vehicula, quam sed ultrices gravida, felis odio iaculis ipsum.[/saas-feature]
[saas-feature title="' . __('Feature Set 3', 'saas') . '" image="{templateUrl}/images/i_clock.png"]Lorem ipsum dolor sit amet, consectetur adipiscing elit. Pellentesque vehicula, quam sed ultrices gravida, felis odio iaculis ipsum.[/saas-feature]
[saas-feature title="' . __('Feature Set 4', 'saas') . '" image="{templateUrl}/images/i_buoy.png"]Lorem ipsum dolor sit amet, consectetur adipiscing elit. Pellentesque vehicula, quam sed ultrices gravida, felis odio iaculis ipsum.[/saas-feature]',
			'homeTestimonials' => '[saas-testimonials title="' . __('What Our Customers Say!', 'saas') . '"]
[saas-testimonial image="{templateUrl}/images/avatar.png" author="' . __('Lyle Hornblower, Roanoke Colony, VA', 'saas') . '"]Donec quis elementum eros. Nam nibh felis, venenatis ut vestibulum a, blandit at arcu. Etiam velit dolor, condimentum at imperdiet ut.[/saas-testimonial]
[/saas-testimonials]',
			'homeFreeTrial' => '[saas-free-trial header="' . __('Free 30-Day Trial', 'saas') . '" footer="' . __('Takes 4 seconds!', 'saas') . '"]
[saas-button type="anchor" url="{baseUrl}/sign-up" text="' . __('Sign Up Now!', 'saas') . '"]
[/saas-free-trial]',
			'pricingGridLayout' => '5',
			'pricingGridContent' => '[saas-grid-column title="' . __('Student', 'saas') . '" price="' . __('Free', 'saas') . '" link_url="{baseUrl}/sign-up/?plan=' . __('Student', 'saas') . '"]
[saas-grid-option text1="1" text2="' . __('Users', 'saas') . '"]
[saas-grid-option text1="1GB" text2="' . __('Space', 'saas') . '" tooltip_title="' . __('Available Space', 'saas') . '" tooltip_text="Duis volutpat nisl ac ante pulvinar non lobortis urna adipiscing. In hac habitasse platea dictumst"]
[saas-grid-option text1="300" text2="' . __('Photos', 'saas') . '"]
[saas-grid-option text1="1GB" text2="' . __('Bandwidth', 'saas') . '" tooltip_title="' . __('Available Bandwidth', 'saas') . '" tooltip_text="Duis volutpat nisl ac ante pulvinar non lobortis urna adipiscing. In hac habitasse platea dictumst"]
[/saas-grid-column]

[saas-grid-column title="' . __('Basic', 'saas') . '" currency="$" price="19" period="' . __('Monthly', 'saas') . '" link_url="{baseUrl}/sign-up/?plan=' . __('Basic', 'saas') . '"]
[saas-grid-option text1="20" text2="' . __('Users', 'saas') . '"]
[saas-grid-option text1="100GB" text2="' . __('Space', 'saas') . '" tooltip_title="' . __('Available Space', 'saas') . '" tooltip_text="Duis volutpat nisl ac ante pulvinar non lobortis urna adipiscing. In hac habitasse platea dictumst"]
[saas-grid-option text1="3000" text2="' . __('Photos', 'saas') . '"]
[saas-grid-option text1="300GB" text2="' . __('Bandwidth', 'saas') . '" tooltip_title="' . __('Available Bandwidth', 'saas') . '" tooltip_text="Duis volutpat nisl ac ante pulvinar non lobortis urna adipiscing. In hac habitasse platea dictumst"]
[/saas-grid-column]

[saas-grid-column highlighted="yes" title="' . __('Professional', 'saas') . '" best_value="' . __('BEST VALUE', 'saas') . '" currency="$" price="25" period="' . __('Monthly', 'saas') . '" link_url="{baseUrl}/sign-up/?plan=' . __('Professional', 'saas') . '"]
[saas-grid-option text1="40" text2="' . __('Users', 'saas') . '"]
[saas-grid-option text1="200GB" text2="' . __('Space', 'saas') . '" tooltip_title="' . __('Available Space', 'saas') . '" tooltip_text="Duis volutpat nisl ac ante pulvinar non lobortis urna adipiscing. In hac habitasse platea dictumst"]
[saas-grid-option text1="5000" text2="' . __('Photos', 'saas') . '"]
[saas-grid-option text1="500GB" text2="' . __('Bandwidth', 'saas') . '" tooltip_title="' . __('Available Bandwidth', 'saas') . '" tooltip_text="Duis volutpat nisl ac ante pulvinar non lobortis urna adipiscing. In hac habitasse platea dictumst"]
[/saas-grid-column]

[saas-grid-column title="' . __('Premier', 'saas') . '" currency="$" price="40" period="' . __('Monthly', 'saas') . '" link_url="{baseUrl}/sign-up/?plan=' . __('Premier', 'saas') . '"]
[saas-grid-option text1="&infin;" text2="' . __('Users', 'saas') . '"]
[saas-grid-option text1="600GB" text2="' . __('Space', 'saas') . '" tooltip_title="' . __('Available Space', 'saas') . '" tooltip_text="Duis volutpat nisl ac ante pulvinar non lobortis urna adipiscing. In hac habitasse platea dictumst"]
[saas-grid-option text1="10,000" text2="' . __('Photos', 'saas') . '"]
[saas-grid-option text1="800GB" text2="' . __('Bandwidth', 'saas') . '"]
[/saas-grid-column]

[saas-grid-column title="' . __('Ultimate', 'saas') . '" currency="$" price="65" period="' . __('Monthly', 'saas') . '" link_url="{baseUrl}/sign-up/?plan=' . __('Ultimate', 'saas') . '"]
[saas-grid-option text1="&infin;" text2="' . __('Users', 'saas') . '"]
[saas-grid-option text1="&infin;" text2="' . __('Space', 'saas') . '" tooltip_title="' . __('Available Space', 'saas') . '" tooltip_text="Duis volutpat nisl ac ante pulvinar non lobortis urna adipiscing. In hac habitasse platea dictumst"]
[saas-grid-option text1="&infin;" text2="' . __('Photos', 'saas') . '"]
[saas-grid-option text1="&infin;" text2="' . __('Bandwidth', 'saas') . '"]
[/saas-grid-column]',
			'footerContent' => '[saas-footer-links1]
[saas-social-links]
[saas-footer-links2]
[saas-footer-links3]',
			'footerLinks1' => '[saas-heading text="' . __('Company', 'saas') . '"]
[saas-link url="#" text="' . __('Leadership', 'saas') . '"] [saas-link url="#" text="' . __('Partners', 'saas') . '"] [saas-link url="#" text="' . __('Investors', 'saas') . '"]',
			'footerLinks2' => '[saas-heading text="' . __('Support', 'saas') . '"]
[saas-link url="#" text="' . __('Create a Support Ticket', 'saas') . '"] [saas-link url="#" text="' . __('FAQs', 'saas') . '"] [saas-link url="#" text="' . __('Forum', 'saas') . '"] [saas-link url="#" text="' . __('Live Chat', 'saas') . '"]',
			'footerLinks3' => '[saas-heading text="' . __('SaaS Corp, Inc.', 'saas') . '"]

' . __('544 Oenoke Ridge', 'saas') . '
' . __('New Canaan, CT 06480', 'saas') . '

[saas-link url="http://gothemeteam.com/" text="http://gothemeteam.com/"]
' . __('Toll Free', 'saas') . ': (800) 555-1212',
			'socialTitle' => __('Keep in Touch', 'saas'),
			'socialTwitter' => 'themeteam',
			'socialFacebook' => 'facebook-name',
			'socialFlickr' => 'flickr-name',
			'socialRss' => get_bloginfo_rss('rss2_url'),
			'socialLinkedin' => 'linkedin-name',
			'socialYoutube' => 'youtube-name',
			'contactInformation' => '[saas-contact-information title="' . __('Contact Information', 'saas') . '"]
[saas-contact-line title="' . __('Toll Free', 'saas') . '"]800-555-1212[/saas-contact-line]
[saas-contact-line title="' . __('Fax', 'saas') . '"]800-555-1212[/saas-contact-line]
[saas-contact-line title="' . __('Address', 'saas') . '"]' . __('544 Oenoke Ridge', 'saas') . '
' . __('New Canaan, CT 06480', 'saas') . '

[saas-link class="link-btn" url="#" text="' . __('View Map', 'saas') . '"][/saas-contact-line]
[/saas-contact-information]',
			'tryUsUrl' => '{baseUrl}/sign-up',
			'tryUsText' => __('Try us free!', 'saas')
		);
		
		$baseUrl = get_bloginfo('url');
		$templateUrl = get_template_directory_uri();
		
		foreach ($defaults as $name => $value)
		{
			if (strpos($value, '{baseUrl}') !== false)
			{
				$value = str_replace('{baseUrl}', $baseUrl, $value);
			}
			
			if (strpos($value, '{templateUrl}') !== false)
			{
				$value = str_replace('{templateUrl}', $templateUrl, $value);
			}
			
			$this->_options->setOption($name, $value);
		}
		
		$this->_options->save();
	}
	
	/**
	 * Performs theme upgrade
	 *
	 * @return void
	 */
	protected function _upgrade()
	{
		$this->_options
			->setOption('version', self::VERSION)
			->save();
	}
	
	/**
	 * Adds menu navigation items for this theme
	 *
	 * @return void
	 */
	public function registerOptionsPages()
	{
		$groupFile = self::HOOK . '-homepage';
		add_object_page(__('General Options', 'saas'), __('SaaS I Theme', 'saas'), 'manage_options', $groupFile, array($this, 'renderHomepageOptions'));
		
		// Homepage
		$page = add_submenu_page($groupFile, __('Homepage Options', 'saas'), __('Homepage Options', 'saas'), 'manage_options', $groupFile, array($this, 'renderHomepageOptions'));
		add_action('admin_print_scripts-' . $page, array($this, 'enqueueAdminScripts'));
		add_action('admin_print_styles-' . $page, array($this, 'enqueueAdminStyles'));
		
		// Pricing Grid
		$page = add_submenu_page($groupFile, __('Pricing Grid Options', 'saas'), __('Pricing Grid Options', 'saas'), 'manage_options', self::HOOK . '-pricing-grid', array($this, 'renderPricingGridOptions'));
		add_action('admin_print_scripts-' . $page, array($this, 'enqueueAdminScripts'));
		add_action('admin_print_styles-' . $page, array($this, 'enqueueAdminStyles'));
		
		// General Options
		$page = add_submenu_page($groupFile, __('General Options', 'saas'), __('General Options', 'saas'), 'manage_options', self::HOOK . '-general', array($this, 'renderGeneralOptions'));
		add_action('admin_print_scripts-' . $page, array($this, 'enqueueAdminScripts'));
		add_action('admin_print_styles-' . $page, array($this, 'enqueueAdminStyles'));
		
		// Footer
		$page = add_submenu_page($groupFile, __('Footer Options', 'saas'), __('Footer Options', 'saas'), 'manage_options', self::HOOK . '-footer', array($this, 'renderFooterOptions'));
		add_action('admin_print_scripts-' . $page, array($this, 'enqueueAdminScripts'));
		add_action('admin_print_styles-' . $page, array($this, 'enqueueAdminStyles'));
	}
	
	/**
	 * Enqueues any javascript needed for this theme
	 *
	 * @return void
	 */
	public function enqueueAdminScripts()
	{
		wp_enqueue_script('postbox');
		wp_enqueue_script('dashboard');
		wp_enqueue_script('jquery-ui-draggable');
		wp_enqueue_script('jquery-ui-droppable');
	}
	
	/**
	 * Enqueues CSS files required for this theme
	 *
	 * @return void
	 */
	public function enqueueAdminStyles()
	{
		wp_enqueue_style(self::HOOK . '-admin-css', get_bloginfo('template_directory') . '/css/saas-admin.css');
	}
	
	/**
	 * Protection for Save Options hooks
	 *
	 * @return boolean
	 */
	protected function _checkSaveOptions()
	{
		if (!isset($_POST['action']) || 'save' != $_POST['action'])
		{
			return;
		}
		
		check_admin_referer(self::OPTIONS_KEY);
		
		if (!current_user_can('manage_options'))
		{
			exit(__('You cannot change SaaS options.', 'saas'));
		}
		
		return true;
	}
	
	/**
	 * Save the Homepage Options
	 *
	 * @return void
	 */
	public function saveHomepageOptions()
	{
		if ($this->_checkSaveOptions())
		{
			// Save options
			$this->_options
				->setOption('homeMessageTitle', filter_input(INPUT_POST, 'homeMessageTitle'))
				->setOption('homeMessageButton', filter_input(INPUT_POST, 'homeMessageButton'))
				->setOption('homeMessageContent', filter_input(INPUT_POST, 'homeMessageContent'))
				->setOption('homeFeaturedImages', filter_input(INPUT_POST, 'homeFeaturedImages'))
				->setOption('homeContentArea', filter_input(INPUT_POST, 'homeContentArea'))
				->setOption('homeTestimonials', filter_input(INPUT_POST, 'homeTestimonials'))
				->setOption('homeFreeTrial', filter_input(INPUT_POST, 'homeFreeTrial'))
				->save();
			
			// Render the message
			$this->_view->messageHelper(__('Homepage Options have been saved.', 'saas'));
		}
	}
	
	/**
	 * Render the Homepage options.
	 * Fires saveHomepageOptions action hook.
	 *
	 * @return void
	 */
	public function renderHomepageOptions()
	{
		do_action(self::HOOK . '-saveHomepageOptions');
		
		// View setup and render
		$this->_view
			->setTemplate('options-homepage')
			->assign('heading', __('SaaS I Theme Homepage Options', 'saas'))
			->assign('homeMessageTitle', $this->_options->getOption('homeMessageTitle'))
			->assign('homeMessageButton', $this->_options->getOption('homeMessageButton'))
			->assign('homeMessageContent', $this->_options->getOption('homeMessageContent'))
			->assign('homeFeaturedImages', $this->_options->getOption('homeFeaturedImages'))
			->assign('homeContentArea', $this->_options->getOption('homeContentArea'))
			->assign('homeTestimonials', $this->_options->getOption('homeTestimonials'))
			->assign('homeFreeTrial', $this->_options->getOption('homeFreeTrial'))
			->assign('onceAction', self::OPTIONS_KEY)
			->render();
	}
	
	/**
	 * Save the Pricing Grid Options
	 *
	 * @return void
	 */
	public function savePricingGridOptions()
	{
		if ($this->_checkSaveOptions())
		{
			// Save options
			$this->_options
				->setOption('pricingGridLayout', (int) filter_input(INPUT_POST, 'pricingGridLayout', FILTER_VALIDATE_INT))
				->setOption('pricingGridContent', filter_input(INPUT_POST, 'pricingGridContent'))
				->save();
			
			// Render the message
			$this->_view->messageHelper(__('Pricing Grid Options have been saved.', 'saas'));
		}
	}
	
	/**
	 * Render the Pricing Grid options.
	 * Fires saveHomepageOptions action hook.
	 *
	 * @return void
	 */
	public function renderPricingGridOptions()
	{
		do_action(self::HOOK . '-savePricingGridOptions');
		
		// View setup and render
		$this->_view
			->setTemplate('options-pricing-grid')
			->assign('heading', __('SaaS I Theme Pricing Grid Options', 'saas'))
			->assign('pricingGridLayouts', array(3, 4, 5))
			->assign('pricingGridLayout', $this->_options->getOption('pricingGridLayout'))
			->assign('pricingGridContent', $this->_options->getOption('pricingGridContent'))
			->assign('onceAction', self::OPTIONS_KEY)
			->render();
	}
	
	/**
	 * Save the General Options
	 *
	 * @return void
	 */
	public function saveGeneralOptions()
	{
		if ($this->_checkSaveOptions())
		{
			// Save options
			$this->_options
				->setOption('colorScheme', filter_input(INPUT_POST, 'colorScheme', FILTER_SANITIZE_SPECIAL_CHARS))
				->setOption('socialTitle', filter_input(INPUT_POST, 'socialTitle', FILTER_SANITIZE_SPECIAL_CHARS))
				->setOption('socialTwitter', filter_input(INPUT_POST, 'socialTwitter', FILTER_SANITIZE_SPECIAL_CHARS))
				->setOption('socialFacebook', filter_input(INPUT_POST, 'socialFacebook', FILTER_SANITIZE_SPECIAL_CHARS))
				->setOption('socialFlickr', filter_input(INPUT_POST, 'socialFlickr', FILTER_SANITIZE_SPECIAL_CHARS))
				->setOption('socialRss', filter_input(INPUT_POST, 'socialRss', FILTER_SANITIZE_URL))
				->setOption('socialLinkedin', filter_input(INPUT_POST, 'socialLinkedin', FILTER_SANITIZE_SPECIAL_CHARS))
				->setOption('socialYoutube', filter_input(INPUT_POST, 'socialYoutube', FILTER_SANITIZE_SPECIAL_CHARS))
				->setOption('tryUsUrl', filter_input(INPUT_POST, 'tryUsUrl'))
				->setOption('tryUsText', filter_input(INPUT_POST, 'tryUsText'))
				->setOption('contactInformation', filter_input(INPUT_POST, 'contactInformation'))
				->save();
			
			// Render the message
			$this->_view->messageHelper(__('General Options have been saved.', 'saas'));
		}
	}
	
	/**
	 * Render the General Options.
	 * Fires saveGeneralOptions action hook.
	 *
	 * @return void
	 */
	public function renderGeneralOptions()
	{
		do_action(self::HOOK . '-saveGeneralOptions');
		
		// View setup and render
		$this->_view
			->setTemplate('options-general')
			->assign('heading', __('SaaS I Theme General Options', 'saas'))
			->assign('colorSchemes', $this->getSupportedColorSchemes())
			->assign('colorScheme', $this->_options->getOption('colorScheme'))
			->assign('socialAccounts', $this->getSupportedSocialAccounts())
			->assign('socialTitle', $this->_options->getOption('socialTitle'))
			->assign('socialTwitter', $this->_options->getOption('socialTwitter'))
			->assign('socialFacebook', $this->_options->getOption('socialFacebook'))
			->assign('socialFlickr', $this->_options->getOption('socialFlickr'))
			->assign('socialRss', $this->_options->getOption('socialRss'))
			->assign('socialLinkedin', $this->_options->getOption('socialLinkedin'))
			->assign('socialYoutube', $this->_options->getOption('socialYoutube'))
			->assign('contactInformation', $this->_options->getOption('contactInformation'))
			->assign('tryUsUrl', $this->_options->getOption('tryUsUrl'))
			->assign('tryUsText', $this->_options->getOption('tryUsText'))
			->assign('onceAction', self::OPTIONS_KEY)
			->render();
	}
	
	/**
	 * Save the Footer Options
	 *
	 * @return void
	 */
	public function saveFooterOptions()
	{
		if ($this->_checkSaveOptions())
		{
			// Save options
			$this->_options
				->setOption('footerContent', filter_input(INPUT_POST, 'footerContent'))
				->setOption('footerLinks1', filter_input(INPUT_POST, 'footerLinks1'))
				->setOption('footerLinks2', filter_input(INPUT_POST, 'footerLinks2'))
				->setOption('footerLinks3', filter_input(INPUT_POST, 'footerLinks3'))
				->save();
			
			// Render the message
			$this->_view->messageHelper(__('Footer Options have been saved.', 'saas'));
		}
	}
	
	/**
	 * Render the Footer Options.
	 * Fires saveFooterOptions action hook.
	 *
	 * @return void
	 */
	public function renderFooterOptions()
	{
		do_action(self::HOOK . '-saveFooterOptions');
		
		// View setup and render
		$this->_view
			->setTemplate('options-footer')
			->assign('heading', __('SaaS I Theme Footer Options', 'saas'))
			->assign('footerContent', $this->_options->getOption('footerContent'))
			->assign('footerLinks1', $this->_options->getOption('footerLinks1'))
			->assign('footerLinks2', $this->_options->getOption('footerLinks2'))
			->assign('footerLinks3', $this->_options->getOption('footerLinks3'))
			->assign('onceAction', self::OPTIONS_KEY)
			->render();
	}
	
	/**
	 * Return a list of supported color schemes
	 *
	 * @return array
	 */
	public function getSupportedColorSchemes()
	{
		$colorSchemes = array
		(
			'default' => __('Default', 'saas'),
			'charcoal' => __('Charcoal', 'saas'),
			'sand' => __('Sand', 'saas'),
			'green' => __('Green', 'saas'),
			'sea-blue' => __('Sea Blue', 'saas'),
			'red-sand' => __('Red Sand', 'saas')
		);
		
		return $colorSchemes;
	}
	
	/**
	 * Return a list of supported social accounts
	 *
	 * @return array
	 */
	public function getSupportedSocialAccounts()
	{
		$socialAccounts = array
		(
			'socialTwitter' => array('name' => __('Twitter Name', 'saas'), 'url' => 'http://twitter.com/[name]'),
			'socialFacebook' => array('name' => __('Facebook Name', 'saas'), 'url' => 'http://www.facebook.com/[name]'),
			'socialFlickr' => array('name' => __('Flickr Name', 'saas'), 'url' => 'http://www.flickr.com/photos/[name]/'),
			'socialRss' => array('name' => __('Rss Url', 'saas')),
			'socialLinkedin' => array('name' => __('LinkedIn Name', 'saas'), 'url' => 'http://linkedin.com/[name]'),
			'socialYoutube' => array('name' => __('Youtube Name', 'saas'), 'url' => 'http://www.youtube.com/user/[name]')
		);
		
		return $socialAccounts;
	}
	
	/**
	 * Renders theme information into the admin footer
	 *
	 * @return void
	 */
	public function renderAdminFooter()
	{
		$this->_view
			->setTemplate('admin-footer')
			->assign('themeHref', 'http://saas-wp-i.worryfreelabs.com/')
			->assign('themeText', __('SaaS I', 'saas'))
			->assign('themeVersion', self::VERSION)
			->assign('authorHref', 'http://themeforest.net/user/goThemeTeam')
			->assign('authorText', 'Theme Team')
			->render();
	}
	
	/**
	 * Returns a value of the theme option identified by $optionName
	 *
	 * @param string $optionName The name of option
	 * @return mixed
	 */
	public static function getThemeOption($optionName)
	{
		return self::getInstance()->_options->getOption($optionName);
	}
	
	/**
	 * Processes the shortcode containing plain text
	 *
	 * @param string $code
	 * @return string
	 */
	protected function _shortcodePlainText($optionName)
	{
		$result = $this->_view
			->setTemplate('shortcode-plain-text')
			->assign('content', $this->_options->getOption($optionName))
			->fetch();
		
		return $result;
	}
	
	/**
	 * Processes the [saas-button] shortcode
	 *
	 * @param array $attr
	 * @return string
	 */
	public function shortcodeButton($atts)
	{
		$defaults = array
		(
			'type' => 'button',
			'url' => null,
			'text' => null
		);
		
		extract(shortcode_atts($defaults, $atts));
		
		$result = $this->_view
			->setTemplate('shortcode-button', true)
			->assign('type', $type)
			->assign('url', $url)
			->assign('text', $text)
			->fetch();
		
		return $result;
	}
	
	/**
	 * Processes the [saas-link] shortcode
	 *
	 * @param array $attr
	 * @return string
	 */
	public function shortcodeLink($atts)
	{
		$defaults = array
		(
			'class' => null,
			'url' => null,
			'text' => null
		);
		
		extract(shortcode_atts($defaults, $atts));
		
		$result = $this->_view
			->setTemplate('shortcode-link', true)
			->assign('class', $class)
			->assign('url', $url)
			->assign('text', $text)
			->fetch();
		
		return $result;
	}
	
	/**
	 * Processes the [saas-heading] shortcode
	 *
	 * @param array $attr
	 * @return string
	 */
	public function shortcodeHeading($atts)
	{
		$defaults = array
		(
			'text' => null
		);
		
		extract(shortcode_atts($defaults, $atts));
		
		$result = $this->_view
			->setTemplate('shortcode-heading', true)
			->assign('text', $text)
			->fetch();
		
		return $result;
	}
	
	/**
	 * Processes the [saas-home] shortcode
	 *
	 * @return string
	 */
	public function shortcodeHome()
	{
		$result = $this->_view
			->setTemplate('shortcode-home')
			->assign('messageTitle', $this->_options->getOption('homeMessageTitle'))
			->assign('messageButton', $this->_options->getOption('homeMessageButton'))
			->assign('messageContent', $this->_options->getOption('homeMessageContent'))
			->assign('featuredImages', $this->_options->getOption('homeFeaturedImages'))
			->assign('contentArea', $this->_options->getOption('homeContentArea'))
			->assign('testimonials', $this->_options->getOption('homeTestimonials'))
			->assign('freeTrial', $this->_options->getOption('homeFreeTrial'))
			->fetch();
		
		return $result;
	}
	
	/**
	 * Processes the [saas-featured-image] shortcode
	 *
	 * @param array $attr
	 * @return void
	 */
	public function shortcodeFeaturedImage($atts)
	{
		$defaults = array
		(
			'title' => null,
			'image' => null
		);
		
		$params = shortcode_atts($defaults, $atts);
		
		$features = $this->_view->featuredImages;
		
		if (!is_array($features))
		{
			$features = array();
		}
		
		array_push($features, $params);
		
		$this->_view->assign('featuredImages', $features);
	}
	
	/**
	 * Processes the [saas-testimonials] shortcode
	 *
	 * @param array $attr
	 * @param mixed $content
	 * @return string
	 */
	public function shortcodeTestimonials($atts, $content = null)
	{
		$defaults = array
		(
			'title' => null
		);
		
		extract(shortcode_atts($defaults, $atts));
		
		$result = $this->_view
			->setTemplate('shortcode-testimonials', true)
			->assign('title', $title)
			->assign('content', $content)
			->fetch();
		
		return $result;
	}
	
	/**
	 * Processes the [saas-testimonial] shortcode
	 *
	 * @param array $attr
	 * @param mixed $content
	 * @return string
	 */
	public function shortcodeTestimonial($atts, $content = null)
	{
		$defaults = array
		(
			'image' => null,
			'author' => null
		);
		
		extract(shortcode_atts($defaults, $atts));
		
		$result = $this->_view
			->setTemplate('shortcode-testimonial', true)
			->assign('image', $image)
			->assign('author', $author)
			->assign('content', $content)
			->fetch();
		
		return $result;
	}
	
	/**
	 * Processes the [saas-free-trial] shortcode
	 *
	 * @param array $attr
	 * @param mixed $content
	 * @return string
	 */
	public function shortcodeFreeTrial($atts, $content = null)
	{
		$defaults = array
		(
			'header' => null,
			'footer' => null
		);
		
		extract(shortcode_atts($defaults, $atts));
		
		$result = $this->_view
			->setTemplate('shortcode-free-trial')
			->assign('header', $header)
			->assign('footer', $footer)
			->assign('content', $content)
			->fetch();
		
		return $result;
	}
	
	/**
	 * Processes the [saas-features] shortcode
	 *
	 * @param array $attr
	 * @param mixed $content
	 * @return string
	 */
	public function shortcodeFeatures($atts, $content = null)
	{
		$defaults = array
		(
			'title' => null
		);
		
		extract(shortcode_atts($defaults, $atts));
		
		$result = $this->_view
			->setTemplate('shortcode-features', true)
			->assign('title', $title)
			->assign('content', strip_tags($content))
			->fetch();
		
		return $result;
	}
	
	/**
	 * Processes the [saas-feature] shortcode
	 *
	 * @param array $attr
	 * @param mixed $content
	 * @return string
	 */
	public function shortcodeFeature($atts, $content = null)
	{
		$defaults = array
		(
			'title' => null,
			'url' => null,
			'image' => null
		);
		
		extract(shortcode_atts($defaults, $atts));
		
		$result = $this->_view
			->setTemplate('shortcode-feature', true)
			->assign('title', $title)
			->assign('url', $url)
			->assign('image', $image)
			->assign('content', $content)
			->fetch();
		
		return $result;
	}
	
	/**
	 * Processes the [saas-footer-links1] shortcode
	 *
	 * @return string
	 */
	public function shortcodeFooterLinks1()
	{
		return $this->_shortcodePlainText('footerLinks1');
	}
	
	/**
	 * Processes the [saas-footer-links2] shortcode
	 *
	 * @return string
	 */
	public function shortcodeFooterLinks2()
	{
		return $this->_shortcodePlainText('footerLinks2');
	}
	
	/**
	 * Processes the [saas-footer-links1] shortcode
	 *
	 * @return string
	 */
	public function shortcodeFooterLinks3()
	{
		return $this->_shortcodePlainText('footerLinks3');
	}
	
	/**
	 * Processes the [saas-social-links] shortcode
	 *
	 * @return string
	 */
	public function shortcodeSocialLinks()
	{
		$socialAccounts = $this->getSupportedSocialAccounts();
		
		$this->_view
			->setTemplate('shortcode-social-links')
			->assign('socialTitle', $this->_options->getOption('socialTitle'))
			->assign('socialAccounts', $socialAccounts);
			
		foreach ($socialAccounts as $code => $data)
		{
			$this->_view->assign($code, $this->_options->getOption($code));
		}
		
		return $this->_view->fetch();
	}
	
	/**
	 * Processes the [saas-contact-information] shortcode
	 *
	 * @param array $atts
	 * @param mixed $content
	 * @return string
	 */
	public function shortcodeContactInformation($atts, $content = null)
	{
		$defaults = array
		(
			'title' => 'no'
		);
		
		extract(shortcode_atts($defaults, $atts));
		
		$result = $this->_view
			->setTemplate('shortcode-contact-information')
			->assign('title', $title)
			->assign('content', $content)
			->fetch();
		
		return $result;
	}
	
	/**
	 * Processes the [saas-contact-line] shortcode
	 *
	 * @param array $atts
	 * @param mixed $content
	 * @return string
	 */
	public function shortcodeContactLine($atts, $content = null)
	{
		$defaults = array
		(
			'title' => 'no'
		);
		
		extract(shortcode_atts($defaults, $atts));
		
		$result = $this->_view
			->setTemplate('shortcode-contact-line')
			->assign('title', $title)
			->assign('content', $content)
			->fetch();
		
		return $result;
	}
	
	/**
	 * Processes the [saas-try-us] shortcode
	 *
	 * @return string
	 */
	public function shortcodeTryUs()
	{
		$result = $this->_view
			->setTemplate('shortcode-try-us')
			->assign('url', $this->_options->getOption('tryUsUrl'))
			->assign('text', $this->_options->getOption('tryUsText'))
			->fetch();
		
		return $result;
	}
	
	/**
	 * Processes the [saas-pricing-grid] shortcode
	 *
	 * @return string
	 */
	public function shortcodePricingGrid()
	{
		$result = $this->_view
			->setTemplate('shortcode-pricing-grid')
			->assign('gridLayout', $this->_options->getOption('pricingGridLayout'))
			->assign('gridContent', $this->_options->getOption('pricingGridContent'))
			->fetch();
		
		return $result;
	}
	
	/**
	 * Processes the [saas-grid-column] shortcode
	 *
	 * @param array $atts
	 * @param mixed $optionsContent
	 * @return string
	 */
	public function shortcodeGridColumn($atts, $optionsContent = null)
	{
		$defaults = array
		(
			'highlighted' => 'no',
			'title'       => null,
			'best_value'  => null,
			'currency'    => null,
			'price'       => null,
			'period'      => null,
			'link_url'    => null,
			'link_text'   => __('Select Plan', 'saas')
		);
		
		extract(shortcode_atts($defaults, $atts));
		
		$result = $this->_view
			->setTemplate('shortcode-grid-section')
			->assign('highlighted', $highlighted)
			->assign('title', $title)
			->assign('bestValue', $best_value)
			->assign('currency', $currency)
			->assign('price', $price)
			->assign('period', $period)
			->assign('linkUrl', $link_url)
			->assign('linkText', $link_text)
			->assign('optionsContent', $optionsContent)
			->fetch();
		
		return $result;
	}
	
	/**
	 * Processes the [saas-grid-option] shortcode
	 *
	 * @param array $atts
	 * @return string
	 */
	public function shortcodeGridOption($atts)
	{
		$defaults = array
		(
			'text1'         => null,
			'text2'         => null,
			'tooltip_title' => null,
			'tooltip_text'  => null,
		);
		
		extract(shortcode_atts($defaults, $atts));
		
		$result = $this->_view
			->setTemplate('shortcode-grid-option', true)
			->assign('text1', $text1)
			->assign('text2', $text2)
			->assign('tooltipTitle', $tooltip_title)
			->assign('tooltipText', $tooltip_text)
			->fetch();
		
		return $result;
	}
	
	/**
	 * Processes the [saas-tour-description] shortcode
	 *
	 * @param array $attr
	 * @param mixed $content
	 * @return string
	 */
	public function shortcodeTourDescription($atts, $content = null)
	{
		$defaults = array
		(
			'title' => null
		);
		
		extract(shortcode_atts($defaults, $atts));
		
		$result = $this->_view
			->setTemplate('shortcode-tour-description', true)
			->assign('title', $title)
			->assign('content', $content)
			->fetch();
		
		return $result;
	}
	
	/**
	 * Processes the [saas-tour-features-list] shortcode
	 *
	 * @param mixed $attr
	 * @param mixed $content
	 * @return string
	 */
	public function shortcodeTourFeaturesList($atts = null, $content = null)
	{
		$result = $this->_view
			->setTemplate('shortcode-tour-features-list', true)
			->assign('content', strip_tags($content))
			->fetch();
		
		return $result;
	}
	
	/**
	 * Processes the [saas-tour-feature] shortcode
	 *
	 * @param array $attr
	 * @param mixed $content
	 * @return string
	 */
	public function shortcodeTourFeature($atts = null, $content = null)
	{
		$defaults = array
		(
			'type' => 'left',
			'url' => null,
			'icon' => null,
			'title' => null
		);
		
		extract(shortcode_atts($defaults, $atts));
		
		$result = $this->_view
			->setTemplate('shortcode-tour-feature', true)
			->assign('type', $type)
			->assign('url', $url)
			->assign('icon', $icon)
			->assign('title', $title)
			->assign('content', $content)
			->fetch();
		
		return $result;
	}
}